unit gtk2icontheme;

{$mode objfpc}{$H+}

interface

{
  Eigenes Programm am GTK+ Icon-Theme anpassen
  Author: DiBo33
  Die Unit ist für jeden frei verfügbar, die Benutzung geschieht auf eigene
  Gefahr und ich übernehme keinerlei Haftung.
}

uses
  Classes, SysUtils, Controls, Graphics, Math,
  gtk2, glib2, gdk2, gdk2pixbuf, gtkdef;

type
  PStrings = array of string;
  
function GetIconRealSize(IconSize: integer): integer;
function GetIconFileName(Name: string; Size: integer): string;
function LoadBitmap(Name: string; Size: integer; bgColor: TColor;
  Bitmap: TBitmap): boolean;
procedure LoadImageList(Images: TImageList; bgColor: TColor; Names: PStrings);
procedure AddIconSearchPath(Path: string);

implementation

const
  Directions: array[0..2] of string[4] = ('', '-ltr', '-rtl');

var
  IconTheme: PGtkIconTheme;
  Picture: TPicture;

{
  GTK+2.2 Funktion, diese ist bereits in FPC definiert allerdings die
  entsprechende Compilerdirective nicht gesetzt, kann dann bei Bedarf
  auskommentiert werden.
}
procedure gdk_draw_pixbuf(drawable: PGdkDrawable; gc: PGdkGC; src: PGdkPixbuf;
  xsrc, ysrc, xdest, ydest, width, height: GInt;
  dither: TGdkRgbDither; xdither, ydither: GInt); cdecl; external gdklib;


function GetIconRealSize(IconSize: integer): integer;
{
  Ermittelt die reale Größe der vorgegebenen Icons.
  Für IconSize sind folgende Werte aus gtkenums.inc möglich
  GTK_ICON_SIZE_MENU = 1
  GTK_ICON_SIZE_SMALL_TOOLBAR = 2
  GTK_ICON_SIZE_LARGE_TOOLBAR = 3
  GTK_ICON_SIZE_BUTTON = 4
  GTK_ICON_SIZE_DND = 5
  GTK_ICON_SIZE_DIALOG = 6
}
var
  w, h: GInt;
begin
  gtk_icon_size_lookup(IconSize, @w, @h);
  Result := w;
end;

function GetIconFileName(Name: string; Size: integer): string;
{
  Ermittelt den Dateinamen eines Icons
  Name: Dateiname ohne Endung 'gtk-open' etc.
  Size: Größe, diese kann mit GetIconRealSize() ermittelt werden.
}
var
  i: integer;
  IconInfo: PGtkIconInfo;
begin
  Result := '';
  i := 0;
  repeat
    IconInfo := gtk_icon_theme_lookup_icon(IconTheme,
      PChar(Name + Directions[i]), Size, GTK_ICON_LOOKUP_USE_BUILTIN);
    inc(i);
  until (IconInfo <> NULL) or (i > High(Directions));
  if IconInfo <> NULL then begin
    Result := gtk_icon_info_get_filename(IconInfo);
    gtk_icon_info_free(IconInfo);
  end;
end;

function LoadThemeIcon(Name: string; Size: integer; bgColor: TColor): boolean;
{
  Icon in Picture.Bitmap laden
  Name: der Dateiname ohne Endung 'gtk-open' etc.
  Size: die gewünschte Größe des Bitmap (Width und Height sind gleich)
  bgColor: Hintergrundfarbe
}
var
  i: integer;
  Pixbuf: PGdkPixbuf;
  Pixmap: PGDIObject;
begin
  Result := false;
  i := 0;
  repeat
    Pixbuf := gtk_icon_theme_load_icon(IconTheme,
      PChar(Name + Directions[i]), Size, GTK_ICON_LOOKUP_FORCE_SVG, NULL);
    inc(i);
  until (Pixbuf <> NULL) or (i > High(Directions));
  if Pixbuf = NULL then Exit;
  Picture.Bitmap.Width := gdk_pixbuf_get_width(Pixbuf);
  Picture.Bitmap.Height := gdk_pixbuf_get_height(Pixbuf);
  Picture.Bitmap.Canvas.Brush.Color := bgColor;
  Picture.Bitmap.Canvas.FillRect(Rect(0, 0, Size, Size));
  Pixmap := PGdiObject(Picture.Bitmap.Handle);
  gdk_draw_pixbuf(Pixmap^.GDIPixmapObject.Image, NULL, Pixbuf, 0, 0, 0, 0,
    Size, Size, GDK_RGB_DITHER_NORMAL, 0, 0);
  gdk_pixbuf_unref(Pixbuf);
  Result := true;
end;

function LoadBitmap(Name: string; Size: integer; bgColor: TColor;
  Bitmap: TBitmap): boolean;
{
  Icon in ein Glyph laden, vorher prüfen ob das Icon geladen werden konnte
  Bitmap: hierin wird das Icon geladen
  Name: der Dateiname ohne Endung 'gtk-open' etc.
  Size: die gewünschte Größe des Bitmap (Width und Height sind gleich)
  Color: Hintergrundfarbe
}
begin
  Result := false;
  if LoadThemeIcon(Name, Size, bgColor) then begin
    Bitmap.Assign(Picture.Bitmap);
    Result := True;
  end;
end;

procedure LoadImageList(Images: TImageList; bgColor: TColor; Names: PStrings);
{
  Icons einer ImageList austauschen
  Images: die ImageList deren Bitmaps ersetzt werden sollen
  Color: Hintergrundfarbe
  Names: ein Array of PChar welches die einzelnen Namen der Icons enthält
}
var
  i, l: integer;
begin
  l := Min(pred(Images.Count), High(Names));
  if l < 1 then Exit;
  for i := 0 to l do if Names[i] <> '' then begin
    if LoadThemeIcon(Names[i], Images.Width, bgColor) then
      Images.Replace(i, Picture.Bitmap, nil);
  end;
end;

procedure AddIconSearchPath(Path: string);
{
  Zusätzlichen Suchpfad der Icons hinzufügen, wichtig für eigene Icons welche
  normal nicht im IconTheme vorhanden sind.
  Path: Die komplette Pfadangabe in dem die Icons gesucht werden sollen.
  Dieser Pfad wird als letztes durchsucht.
}
begin
  gtk_icon_theme_append_search_path(gtk_icon_theme_get_default(), PChar(Path));
end;

initialization
  IconTheme := gtk_icon_theme_get_default();
  Picture := TPicture.Create;
  
finalization
  Picture.Free;
  
end.

